#pragma once

#include "core/utils/utils.h"
#include "core/jobs/jobs.h"
#include "core/improc/map2d.h"

#include "rev_polish_job.h"

/* hariharan job class */
class hariharan : public job_base
{
public:
	// ctor
	hariharan(const std::array<map2d<float>, 5>& fringes)
	{
		// convert fringes to rev_polish_job_base objects
		std::vector<std::shared_ptr<rev_polish_job_base>> fringes_objects;

		for (size_t i = 0; i < 5; i++)
			fringes_objects.push_back(std::make_shared<rev_polish_job_ref>(fringes[i]));

		// create T objects
		auto T1 = create_reverse_polish_job(fringes_objects, "&1 &3 -");
		auto T2 = create_reverse_polish_job(fringes_objects, "&2 &0 - &2 &4 - +");
		auto T3 = create_reverse_polish_job(fringes_objects, "&2 &1 &0 + + &2 &4 &3 + + +");

		std::vector<std::shared_ptr<rev_polish_job_base>> t_objects;

		t_objects.push_back(nullptr);
		t_objects.push_back(T1);
		t_objects.push_back(T2);
		t_objects.push_back(T3);

		// create phase and contrast dependencies
		this->m_phase = create_reverse_polish_job(t_objects, "&2 neg 2 &1 * neg atan2");
		this->m_contrast = create_reverse_polish_job(t_objects, "2 &3 * 3 4 &1 ^2 * &2 ^2 + sqrt * /");

		// add as dependencies
		add_dependency(this->m_phase);
		add_dependency(this->m_contrast);
	}

	// return phase
	const map2d<float>& get_phase(void) const
	{
		if (__ISNULLPTR(this->m_phase))
			throw_exception(null_job_exception);

		return this->m_phase->get();
	}

	// return contrast
	const map2d<float>& get_contrast(void) const
	{
		if (__ISNULLPTR(this->m_contrast))
			throw_exception(null_job_exception);

		return this->m_contrast->get();
	}

private:

	// nothing to do
	virtual void _run(void) override {}

	// these will hold the results
	std::shared_ptr<rev_polish_job_base> m_contrast, m_phase;
};
